package pg.android.kernel;

import java.util.ArrayList;

/**
 * Representiert einen Sensor/Aktor
 */
public class Device {
	// Speicher fr ausgelesene Werte
	private ArrayList<Float> history = new ArrayList<Float>();
	
	protected byte type;	
	protected byte subType;
	
	public int number;
	//Sensor/Aktor Daten
	public String name;
	public byte dataType;
	public int bitDepth;
	public int scale;
	public float minValue;
	public float maxValue;
	private float curValue;
	
	public float disp_min = 0;
	public float disp_max = 0.01f;
	
	public Device (int number, byte deviceType, byte deviceSubType, byte dataType, int bitDepth, int scale, int minValue,int maxValue) {
		this.number = number;
		this.type = deviceType;
		this.subType = deviceSubType;
		this.name = "";
		this.dataType = dataType;
		this.bitDepth = bitDepth;
		this.scale = scale;

		
		this.minValue = (float) Math.pow(10, (scale - 128)) * ((float)minValue - 32767);
		this.maxValue = (float) Math.pow(10, (scale - 128)) * ((float)maxValue - 32767);
	}
	
	/**
	 * Gibt aktuellen Wert zurck
	 * @return aktueller Wert
	 */
	public float getValue () {
		return curValue;
	}
	
	/**
	 * Setzt aktuellen Wert
	 * @param low low-Byte aus dem Byte-Paket
	 * @param high high-Byte aus dem Byte-Paket
	 */
	public void setValue (byte low, byte high){
		int value = (int)(get(high) << 8) | get(low);
		
		float range = maxValue - minValue;
		float bitScale = (float) (Math.pow(2, bitDepth) - 1);
		
		//Wert konvertieren
		curValue = minValue+(value*range/bitScale);
		//Wert runden
		curValue = (float) Math.round(curValue * 100) / 100;
		
		synchronized (history) {
			if (history.size() == 0) {
				if (curValue == minValue){
					disp_max = (float)(curValue + 0.05);
					disp_min = minValue;					
				} else if (curValue == maxValue){
					disp_max = maxValue;
					disp_min = (float)(curValue - 0.05);					
				} else {
					disp_max = (float)(curValue + 0.03);
					disp_min = (float)(curValue - 0.02);					
				}
			}
		}

		if (disp_min > curValue) {
			disp_min = curValue;
		} else if (disp_max < curValue) {
			disp_max = curValue;
		}
		
		synchronized (history) {
			history.add(new Float(curValue));
		}
	}
	
	/**
	 * Gibt minimum zurck
	 * @return
	 */
	public float getMin () {
		return minValue;
	}
	/**
	 * Gibt maximum zurck
	 * @return
	 */
	public float getMax () {
		return maxValue;
	}
	/**
	 * Gibt Range zurck
	 * @return
	 */
	public float getRange() {
		return maxValue - minValue;
	}
	/**
	 * Gibt alle gespeicherte Werte zurck
	 * @return
	 */
	public final ArrayList<Float> getHistory () {
		return history;
	}
	
	/**
	 * Gibt Typ vom Sensor/Aktor zurck
	 * @return
	 */
	public byte getType (){
		return type;
	}

	/**
	 * Gibt Subtyp vom Sensor/Aktor zurck
	 * @return
	 */
	public byte getSubType (){
		return subType;
	}
	
	/**
	 * Konvertiert ein Byte in ein Integer
	 * @param value Byte-Wert
	 * @return konverierter Integer-Wert
	 */
	private static int get (byte value) {
		return 0x000000FF & ((int)value);
	}
	
	/**
	 * Byte-Paket wird zum Device geparst
	 * @param buffer
	 * @return
	 */
	public static Device parse (final byte[] buffer) {
		int number = buffer[4];
		byte deviceType = buffer[5];
		byte deviceSubType = buffer[6];
		byte dataType = buffer[7];
		int bitDepth = get(buffer[8]);
		int scale = get(buffer[9]);

		int minValue = (int)((get(buffer[11]) << 8) | get(buffer[10]));
		int maxValue = (int)((get(buffer[13]) << 8) | get(buffer[12]));

		
        //TODO TEST
		/*
        if (number == 0) {
        	deviceType = (byte)0x82;
        	deviceSubType = (byte)0x02;
        }
		*/
		
		Device device = new Device (number, deviceType, deviceSubType, dataType, bitDepth, scale, minValue, maxValue);
		
		return device;
	}

}
